﻿function el(id) {
	return document.getElementById(id);
}

function formatDate(date) {
	const d = new Date(date);
	if (d.getFullYear() < 1900 || isNaN(d)) { return ''; }

	return ('0' + d.getDate()).slice(-2) + '.' + ('0' + (d.getMonth() + 1)).slice(-2) + '.' + d.getFullYear();
}

function formatNumber(value, fractionSize = 2) {
	const padding = '000000';
	let [integer, fraction = ''] = ('' + value || '').split('.');

	fraction = fractionSize > 0 ? ',' + (fraction + padding).substring(0, fractionSize) : '';
	integer = integer.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
	return integer + fraction;
}

function parseDate(value) {
	const str = value.split('.');
	const day = +str[0];
	const month = +str[1];
	const year = +str[2];

	const d = new Date(year, month-1, day);
	if (isNaN(d) || d.getFullYear() != year || d.getMonth() != month-1 || d.getDate() != day) {
		return '1899-12-30';
	}

	return ('0000' + year).slice(-4) + '-' + ('0' + month).slice(-2) + '-' + ('0' + day).slice(-2);
}

function checkDate(id) {
	const value = el(id).value;
	const iso = parseDate(value);
	const value1 = formatDate(iso);
	el(id).value = value1;
}

function errorMessage(text) {
	el('message').innerText = text;
	el('message').className = 'focloud-error';
	el('form').className = 'd-none';
}

function successMessage(text) {
	el('message').innerText = text;
	el('message').className = 'focloud-info';
	el('form').className = 'd-none';
}

function successMessageID(id, lang) {
	id = (id || '').toLowerCase();
	lang = (lang || '').toLowerCase();

	var messages = [
		{ id: 'saved', lang: 'de', text: 'Die Daten wurden gespeichert.' },
		{ id: 'transok', lang: 'de', text: 'Die Transaktion war erfolgreich.' },
		{ id: 'offer', lang: 'de', text: 'Das Angebot wurde erfolgreich bestätigt.' },
		{ id: 'sent', lang: 'de', text: 'Die Nachricht wurde gesendet.' },

		{ id: 'saved', lang: 'en', text: 'You data has been saved.' },
		{ id: 'transok', lang: 'en', text: 'The transaction was successfull.' },
		{ id: 'offer', lang: 'en', text: 'Your offer was successfully confirmed.' },
		{ id: 'sent', lang: 'en', text: 'Your message was sent successfully.' },

		{ id: 'saved', lang: 'it', text: 'I dati sono stati salvati.' },
		{ id: 'transok', lang: 'it', text: 'La transazione è andata a buon fine.' },
		{ id: 'offer', lang: 'it', text: `L'offerta è stata confermata con successo` },
		{ id: 'sent', lang: 'it', text: 'Il messaggio è stato inviato' },

		{ id: 'saved', lang: 'hu', text: 'Az adatai sikeresen mentésre kerültek.' },
		{ id: 'transok', lang: 'hu', text: 'A tranzakció sikeres volt.' },
		{ id: 'offer', lang: 'hu', text: 'Az ajánlat megerősítése sikeres volt.' },
	];

	var message = messages.find(m => m.id == id && m.lang == lang);
	if (!message) {
		message = messages.find(m => m.id == id.toLowerCase());
	}

	successMessage(message ? message.text : 'Message string ' + id + ' not found.');
}

function getParameterByName(name) {
	if (typeof _config != 'undefined' && _config[name]) {
		return _config[name];
	}

	var url = window.location.href;

	name = name.replace(/[\[\]]/g, '\\$&');
	var regex = new RegExp('[?&]' + name + '(=([^&#]*)|&|#|$)'),
		results = regex.exec(url);

	if (!results) return null;
	if (!results[2]) return '';
	return decodeURIComponent(results[2].replace(/\+/g, ' '));
}


function setLoading(active) {
	var button = el('submit');
	if (button) {
		button.disabled = active;
	}

	var spinner = el('spinner');
	if (spinner) {
		spinner.className = active ? 'd-inline' : 'd-none';
	}
}

function getCode() {
	const qry = window.location.search;
	let code = getParameterByName('code');
	if (!code) {
		code = getParameterByName('refid');
	}

	if (!code && !getParameterByName('db')) {
		code = qry.substr(1);
	}

	return code;
}

async function getEndpoint(code) {
	var local = getParameterByName('local');
	if (local) { return 'http://localhost:2479/api/external/'}


	var db = getParameterByName('db');
	if (!db) {
		var arr;
		try {
			arr = atob(code).split('|');
		} catch (e) {
			return '';
		}
		db = arr[0];
	}

	var host = await getPlain('https://support.sitec.at/endpoint?' + db)
	var endpoint = 'https://' + host + '/' + db + '/api/external/';
	console.log('Endpoint: ' + endpoint);
	return endpoint;
}

async function getPlain(url) {
	return new Promise(async (resolve) => {
		const response = await fetch(url)
			.catch(err => errorMessage(err));

		let result = null;
		if (response.ok) {
			result = await response.text();
		} else {
			const text = await response.text();
			errorMessage(text);
		}
		resolve(result);
	});
};

async function get(url) {
	return new Promise(async (resolve) => {
		const response = await fetch(url)
			.catch(err => errorMessage(err));

		let result = null;
		if (response.ok) {
			result = await response.json();
		} else {
			const text = await response.text();
			errorMessage(text);
		}
		resolve(result);
	});
};

async function put(url, data) {
	return new Promise(async (resolve) => {
		const response = await fetch(url, {
			method: 'PUT',
			body: data,
			headers: {
				'Content-Type': 'application/json'
			}
		})
		.catch(err => errorMessage(err));

		let result = null;
		if (response.ok) {
			result = await response.json();
		} else {
			const text = await response.text();
			errorMessage(text);
		}

		resolve(result);
	});
};

async function post(url, data) {
	return new Promise(async (resolve) => {
		const response = await fetch(url, {
			method: 'POST',
			body: data,
			headers: {
				'Content-Type': 'application/json'
			}
		})
			.catch(err => errorMessage(err));

		let result = null;
		if (response.ok) {
			result = await response.json();
		} else {
			const text = await response.text();
			errorMessage(text);
		}

		resolve(result);
	});
};

////////////////////////
// Main
var css = getParameterByName('css');
if (css) {
	var style = this.document.createElement('link');
	style.rel = 'stylesheet';
	style.href = css;

	var links = this.document.getElementsByTagName('link');
	var link = links[links.length - 1];
	link.parentNode.insertBefore(style, link.nextSibling);
}


////////////////////////
// CheckIn functions
async function filloutCheckInForm(index) {
	if (window.location.search.length < 5) return errorMessage('Ungültiger Buchungscode');

	const code = getCode();
	const endpoint = await getEndpoint(code);
	const data = await get(endpoint + 'checkin/' + code);

	if (!data) return;

	for (var i = 0; i < 6; i++) {
		var row = el('booking_' + i);

		if (i < data.bookings.length) {
			var radio = row.getElementsByTagName('input')[0];
			radio.checked = i == index;

			var text = row.getElementsByTagName('a')[0];
			text.innerText = data.bookings[i].category.name + ' #' + data.bookings[i].room + ': ' + formatDate(data.bookings[i].from) + ' - ' + formatDate(data.bookings[i].to)
				+ ', ' + data.bookings[i].text + ' (' + data.bookings[i].id + ')';

		} else {
			row.classList.add('d-none');
        }
	}

	if (data.bookings.length > 1) {
		el('b_select').style.display = 'inherit';
	}

	const booking = data.bookings[index];
	const adr = data.adrs[index];

	el('form').reset();
	el('form').className = '';

	el('b_id').innerText = booking.id;
	el('b_room').innerText = booking.category.name;
	el('b_dates').innerText = formatDate(booking.from) + ' - ' + formatDate(booking.to);
	el('b_pers').innerText = booking.persCount;
	el('code').value = booking.guid;

	// Adr data
	el('id').value = adr.id;
	el('firstname').disabled = true;
	el('name').disabled = true;


	const o = Object.entries(adr);
	o.forEach(field => {
		let ctrl = el(field[0]);
		if (ctrl !== null) {
			if (field[0].indexOf('date') >= 0) {
				ctrl.value = formatDate(field[1]);
			} else if (ctrl.type === 'checkbox') {
				ctrl.checked = field[1];
			} else {
				ctrl.value = field[1];
			}
		}
	});

	for (var i = 1; i <= 5; i++) {
		el('sub' + i).style.display = 'none';
	}

	for (var i = 1; i <= 5; i++) {
		if (i-1 >= adr.subAdrs.length) break;

		const subAdr = adr.subAdrs[i-1];
		el('subID' + i).value = subAdr.id;
		el('subFirstname' + i).value = subAdr.firstname;
		el('subName' + i).value = subAdr.name;
		el('subBirthdate' + i).value = formatDate(subAdr.birthdate);
		el('subRelation' + i).value = subAdr.relation;

		el('sub' + i).style.display = 'flex';
	}
}


async function setCheckInBooking(index) {
	if (await onSaveCheckInForm() == false) { return; }
	filloutCheckInForm(index);
}

function onNewAdr() {
	el('firstname').disabled = false;
	el('name').disabled = false;
	el('id').value = 0;
	el('form').reset();

	for (var i = 1; i <= 5; i++) {
		el('sub' + i).style.display = 'none';
	}
}

function onChangeAdr() {
	el('firstname').disabled = false;
	el('name').disabled = false;
}


function addSubAdr() {
	for (var i = 1; i <= 5; i++) {
		if (el('sub' + i).style.display === 'none') {
			el('sub' + i).style.display = 'flex';
			return;
		}
	}
}

async function onSaveCheckInForm() {
	const form = el('form');
	if (!form.checkValidity()) {
		form.classList.add('was-validated');
		return false;
	}

	setLoading(true);

	const data = [];
	for (var i = 0; i < form.elements.length; i++) {
		var id = form.elements[i].id;
		data[id] = form.elements[i];
	}

	let adr = {};
	for (const [id, e] of Object.entries(data)) {
		if (!id) { continue; }

		let val = e.value;

		if (e.id.indexOf('date') >= 0)
			val = parseDate(val);

		if (e.id.substr(0, 3) !== 'sub') {
			adr[e.id] = val;
		}

		if (e.type === 'checkbox') {
			adr[e.id] = !!e.checked;
		}
	}

	// subadrs
	adr.subAdrs = [];
	for (let i = 1; i <= 5; i++) {
		if (data['subFirstname' + i].value) {
			const subAdr = {};
			subAdr.id = data['subID' + i].value || '0';
			subAdr.firstname = data['subFirstname' + i].value;
			subAdr.name = data['subName' + i].value;
			subAdr.birthdate = parseDate(data['subBirthdate' + i].value);
			subAdr.relation = data['subRelation' + i].value;
			adr.subAdrs.push(subAdr);
		}
	}

	const code = data['code'].value
	const endpoint = await getEndpoint(code);
	const result = await put(endpoint + 'checkin/' + code, JSON.stringify(adr));
	setLoading(false);

	if (!result) { return false; }

	successMessageID('saved', data['language'].value);
	return true;
}

////////////////////////
// Deposit 
async function filloutDepositForm() {
	var urlParams = new URLSearchParams(window.location.search);
	if (urlParams.get('ERRTEXT') !== null) return errorMessage('Die Transaktion wurde abgebrochen: ' + urlParams.get('ERRTEXT'));
	if (urlParams.get('TID') || urlParams.get('success')) {
		el('form').className = 'd-none';
		return successMessageID('transOk');
	}

	if (window.location.search.length < 5) return errorMessage('Ungültiger Buchungscode');

	const code = getCode();
	const ref = location.href.split('?');
	const endpoint = await getEndpoint(code);
	const data = await get(endpoint + 'deposit/' + code + '?refferer=' + ref[0]);
	if (!data) { return; }

	el('b_text').innerText = "'" + data.booking.text + "'";
	el('from').innerText = formatDate(data.booking.from);
	el('to').innerText = formatDate(data.booking.to);
	el('refid').innerText = data.booking.refID;
	el('arr').innerText = data.booking.arrangementID;

	el('total').innerText = formatNumber(data.deposit.total);
	el('deposit').innerText = formatNumber(data.deposit.deposit);
	el('option').innerText = formatDate(data.deposit.option);

	el('submit').onclick = function () { location.href = data.deposit.url; };
	el('form').className = '';
}

////////////////////////
// Payment 
async function filloutPaymentForm() {
	var urlParams = new URLSearchParams(window.location.search);
	if (urlParams.get('ERRTEXT') !== null) return errorMessage('Die Transaktion wurde abgebrochen: ' + urlParams.get('ERRTEXT'));
	if (urlParams.get('TID') || urlParams.get('success')) {
		el('form').className = 'd-none';
		return successMessageID('transOk');
	}

	if (window.location.search.length < 5) return errorMessage('Ungültiger Code');

	const code = getCode();
	const ref = location.href.split('?');
	const endpoint = await getEndpoint(code);
	const data = await get(endpoint + 'payment/' + code + '?refferer=' + ref[0]);
	if (!data) { return; }

	const invoiceID = +data.invoice.invoiceID;
	if (invoiceID > 0) {
		el('invoiceID').innerText = invoiceID;
		el('b_text').innerText = invoiceID;
	} else {
		el('b_text').innerText = "'" + data.account.text + "'";
	}

	el('date').innerText = formatDate(data.invoice.date);
	el('invID').innerText = data.invoice.invID;
	el('text').innerText = data.account.text;

	el('total').innerText = formatNumber(data.account.total);
	el('balance').innerText = formatNumber(data.account.balance);

	el('submit').onclick = function () { location.href = data.url; };
	el('form').className = '';
}


////////////////////////
// Offer
async function filloutOfferForm() {
	if (window.location.search.length < 5) return errorMessage('Ungültiger Buchungscode');

	const code = getCode();
	const endpoint = await getEndpoint(code);
	const data = await get(endpoint + 'offer/' + code);
	if (!data) { return; }

	el('b_text').innerText = "'" + data.booking.text + "'";
	el('from').innerText = formatDate(data.booking.from);
	el('to').innerText = formatDate(data.booking.to);
	el('bid').innerText = data.booking.id;
	el('refid').innerText = data.booking.refID;
	el('arr').innerText = data.booking.arrangementID;

	el('total').innerText = formatNumber(data.total);
	el('form').className = '';
}

async function onSubmitOfferForm() {
	setLoading(true);

	const code = getCode();
	const endpoint = await getEndpoint(code);
	const response = await get(endpoint + 'acceptoffer/' + code);

	setLoading(false);
	if (!response) { return; }

	successMessageID('offer');
};


////////////////////////
// Unsubscribe
async function filloutSubscriberForm() {
	if (window.location.search.length < 5) return errorMessage('Ungültiger Abmeldecode');

	const code = getCode();
	const endpoint = await getEndpoint(code);
	const data = await get(endpoint + 'subscriber/' + code);
	if (!data) { return; }

	el('submit').onclick = function () { onSubmitSubscriberForm(); };
	el('name').innerText = data.name;
	el('email').innerText = data.email;
	el('form').className = '';
}

async function onSubmitSubscriberForm(e) {
	setLoading(true);

	const code = getCode();
	const endpoint = await getEndpoint(code);
	const result = await get(endpoint + 'unsubscribe/' + code);

	setLoading(false);
	if (!result) { return; }

	successMessage(result.message);
}


////////////////////////
// Voucher
async function filloutVoucherForm() {
	var urlParams = new URLSearchParams(window.location.search);
	if (urlParams.get('ERRTEXT') !== null) return errorMessage('Die Transaktion wurde abgebrochen: ' + urlParams.get('ERRTEXT'));

	if (urlParams.get('TID') || urlParams.get('success')) {
		const code = urlParams.get('code');
		var refID = '';
		try {
			arr = atob(code).split('|');
			refID = arr[2];
		} catch (e) {
		}

		const amount = +urlParams.get('amount');

		el('form').className = 'd-none';
		return successMessage('Der Gutschein ' + refID + ', Betrag ' + amount.toLocaleString(undefined, { minimumFractionDigits: 2 }) + ' wurde erfolgreich erstellt');
	}

	const code = getCode();
	if (!code) { return; }
	const endpoint = await getEndpoint(code);
	const data = await get(endpoint + 'voucher/' + code);
	if (!data) { return; }

	if (data.email) {
		el('email').value = data.email;
		el('addresses').value = data.addresses;
		el('name').value = data.name;
		el('firstname').value = data.firstname;
	}
}

async function onSubmitVoucherForm() {

	var form = el('form');
	if (!form.checkValidity()) {
		event.preventDefault();
		event.stopPropagation();
		form.classList.add('was-validated');
		return;
	}

	var data = {
		amount: Math.floor(el('amount').value.replace(',', '.')),
		text: el('text').value,
		email: el('email').value,
		addresses: el('addresses').value,
		firstname: el('firstname').value,
		name: el('name').value,
		zipcode: el('zipcode').value,
		city: el('city').value,
		street: el('street').value,
	};

	setLoading(true);

	const code = getCode();
	const endpoint = await getEndpoint(code);
	const ref = location.href.split('?');
	const result = await put(endpoint + 'voucher?refferer=' + ref[0], JSON.stringify(data));
	setLoading(false);

	if (!result) { return; }
		
	window.open(result.url, '_blank');
}

////////////////////////
// Chat
async function filloutChatForm() {
	if (window.location.search.length < 5) return errorMessage('Ungültiger Buchungscode');

	const code = getCode();
	const endpoint = await getEndpoint(code);
	const data = await get(endpoint + 'chat/' + code);

	if (!data) return;

	el('salutation').innerHTML = data.salutation + '!';
	el('chatHtml').innerHTML = data.chatHtml;
	el('refID').innerHTML = data.refID;
	el('docLink').href = data.docLink;


	el('form').className = '';
}


async function onSubmitChatForm() {

	var form = el('form');
	if (!form.checkValidity()) {
		event.preventDefault();
		event.stopPropagation();
		form.classList.add('was-validated');
		return;
	}

	var data = {
		subject: `Chat-Formular vom ${new Date().toLocaleDateString()}`,
		text: el('text').value
	}

	const code = getCode();
	const endpoint = await getEndpoint(code);
	const result = await post(endpoint + 'chat/' + code, JSON.stringify(data));

	setLoading(false);
	if (!result) { return false; }

	successMessageID('sent', 'de');
	return true;
}

////////////////////////
// Newsletter
async function onSaveSubscription() {
	const form = el('form');
	if (!form.checkValidity()) {
		form.classList.add('was-validated');
		return false;
	}

	setLoading(true);

	const data = [];
	for (var i = 0; i < form.elements.length; i++) {
		var id = form.elements[i].id;
		data[id] = form.elements[i];
	}

	let adr = {};
	for (const [id, e] of Object.entries(data)) {
		if (!id) { continue; }

		let val = e.value;
		adr[e.id] = val;
	}

	const endpoint = await getEndpoint('').replace('external', 'newsletter');
	const result = await post(endpoint, JSON.stringify(adr));
	setLoading(false);

	if (!result) { return false; }

	successMessageID('saved', data['language'].value);
	return true;
}


/* FO Form include support */
function initFOForms() {
	const z = document.getElementsByTagName('fo-form');
	for (var i = 0; i < z.length; i++) {
		const el = z[i];

		const url = el.getAttribute('href');
		if (url) {
			const xhttp = new XMLHttpRequest();
			xhttp.onreadystatechange = function () {
				if (this.readyState == 4) {
					if (this.status == 200) {
						const html = this.responseText;
						const start = html.indexOf('<body');
						if (start > 0) {
							const startContent = html.indexOf('>', start);
							const end = html.indexOf('</body>');
							el.innerHTML = html.substr(startContent + 1, end - startContent - 1);

							const onload = html.indexOf('onload="');
							if (onload) {
								const onloadEnd = html.indexOf('"', onload + 10);
								const fn = html.substr(onload + 8, onloadEnd - onload - 8);
								eval(fn);
							}
						} else { el.innerHTML = 'Body not found.'; }
					}
					if (this.status == 404) { el.innerHTML = 'Page not found.'; }

					el.removeAttribute('href');
					initFOForms();
				}
			}
			xhttp.open('GET', url, true);
			xhttp.send();
		}
	}
}

initFOForms();
